/*****************************************************************************

    Filename: sb_custom.c
    Date: 13/02/2006
    Revision: 1.0 
    Author: SM 
    
    National Semiconductor Corporation

    Description: 
    Implementation of Simply Blue Interface framework for embedded
    development platform preferably using MicroC/OS-II RTOS.   
    
    The scope of this file is not to give a complete code compilable and 
    runnable, but deliver a framework to start with. The implementation of 
    the embedded application depends on the need and the platform used.
*****************************************************************************/

#include <stdio.h>                         
#include <stdlib.h>
#include <string.h>
#include "drivers.h"      /* CP3000 specific: Drivers for CP3000 products */
#include "dbgmsg.h"       /* Debug interface used with our CP3000 embedded products */

#include "sb_custom.h"    /* Header file for this c file*/
#include "ucos.h"         /* Header file for UCOS functions */

#define SB_UART_PORT      2       /* UART port for debug. To connect to a terminal */

//#define MAX_CONNECTED_DEVICES 3 /* specific LMX9820A */
#define MAX_CONNECTED_DEVICES 7   /* specific LMX9830 */

#define SB_TIMER_TICKS        4   /* N  => (N * 10) ms */

static const usart_conf UsartInitConf =
{                          /* UART parameters for CP3SP33 - module communication */
    USART_XFER_DMA,        /* Use of UART DMA transfer */ 
    BPS_115200,            /* Speed 115200 BPS */
    HW_FLOW_CONTROL,       /* Use of Hardware Flow control */
    10000,                 /* UART timeout = 10000 */
    NULL
};

//Simply Blue supported codec types
typedef enum {
    SBCODEC_NONE,          /* No codec connected */
    SBCODEC_MOTOROLA,      /* Motorola codec MC145483 */
    SBCODEC_OKI,           /* OKI codec MSM7717 (Obsoleted!)*/
    SBCODEC_PCMSLAVE       /* PCM slave mode */
}SbCodecType_T;

//Simply Blue supported air formats
typedef enum {
    SBAIRFMT_CVSD,         /* CVSD compression */
    SBAIRFMT_MULAW,        /* uLAW compression */
    SBAIRFMT_ALAW          /* aLAW compression */
}SbAirFormat_T;


typedef struct 
{
	uint8   bStart;             
	uint8   bType;              
	uint8   bOpcode;
	uint16  wPayloadlen;
	uint16  wPayloadOffset;
	uint8   bChecksum;
	uint8   pPayload[340];
	uint8   bEnd;
} SBEvent_T;

//local device information
typedef struct {
    uint32   PortsToOpen;          /* The port numbers that are opened on the local device */
	OS_EVENT *SbCmdMbox;	       /* Mailbox for Simply Blue commands */
    SbPacketState_T SbPacketState; /* Current Sb packet byte position */
}SbDeviceInfoType;

static SbDeviceInfoType SbDevInfo; /* local device context information */

static SBEvent_T        SbEvent;   /* Current result of Simply Blue response */

static int sbTimerHandle = 0;      /* Timer handle */

Boolean Transparent_flag = FALSE;  /* Transparent flag to manage transparent mode /*

/************ Functions Prototypes *******************************/
static SBStatus_T SbSendCommand(uint8* SbCommand, uint16 Size);
static void       SbProcessFrame(void);
static SBStatus_T SbInit(void);
static SBStatus_T SbEstablishLink(BdAddrType,uint8,uint8);  
static SBStatus_T SbReleaseLink(uint8 LocalPortNo);  
static SBStatus_T SbEnterTransparentMode(uint8 LocalPortNo);
static void       SbSendUartBreak(void);
static boolean    SbSdapStartSession( BdAddrType bdAddr );
static boolean    SbSdapStopSession(void);
static SBStatus_T SbDeleteSdpRecords(void);
static SBStatus_T SbStoreSdpRecord(uint8* entry, uint16 length);
static SBStatus_T SbSetOperationMode(SBOperationMode_T mode);
static void       SbNvsReadBdAddr(BdAddrType BdAddr);
static void       SbNvsWriteBdAddr(BdAddrType BdAddr);
static uint8      SbNvsReadDeviceName(BtNameType, uint8*, uint8);
static void       SbNvsWriteDeviceName(BtNameType Name,uint8 length);
static SBStatus_T SbSetNvsDefaultAudioSettings(uint8, uint8);
static SBStatus_T SbResetFactorySettings(void);
static SBStatus_T SbResetDevice(void);
static SBStatus_T SbConfigureLocalDevice(void);

static void SbTimerTask( void );

//SB framework api
/******************************************************************************  
    Function: SbApiInit

	Description
	-----------
    This function Intializes the following :
    - initializes the simply blue software architecture
    - initializes the usart port that is being used for simply blue connection.
    - initializes the simply blue firmware

	Arguments
	---------
    None   

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbApiInit(void)
{
    /* Create a mailbox for inter-task communication */
	SbDevInfo.SbCmdMbox = OSMboxCreate((void *)0);
    
    if (SbDevInfo.SbCmdMbox == NULL) {
        return SBSTATUS_ERROR;
    }

    SbDevInfo.PortsToOpen     = 0;
    SbDevInfo.SbPacketState   = SBPACKET_START;

    SbEvent.wPayloadOffset = 0;

    sbTimerHandle = timer_set_wakeup( SbTimerTask, SB_TIMER_TICKS, TRUE );
    
    if (sbTimerHandle == -1) {
        return SBSTATUS_NO_TIMER;
    }

    /* Initialize DMA driver for UART DMA use */
    DmaInit();

    /* Initialize UART port communication with SB module */
    usart_init(SB_UART_PORT, &UsartInitConf);

    return SBSTATUS_OK;
}
/******************************************************************************  
    Function: SbInit

	Description
	-----------
    This function Intializes the SimplyBlue module.
    It basically does a restore to factory settings

	Arguments
	---------
    None   

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbInit(void)
{
    return SbConfigureLocalDevice();
}
/******************************************************************************  
    Function: SbApiShutdown

	Description
	-----------
    This function shutsdown the simply blue software system and associated
    hardware peripherals.

	Arguments
	---------
    None   

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbApiShutdown(void)
{
    uint8 err;

    OSMboxDel(SbDevInfo.SbCmdMbox, OS_DEL_ALWAYS, &err);

    timer_clear_wakeup( sbTimerHandle );
    
    return SBSTATUS_OK;
}

/******************************************************************************  
    Function: SbSendCommand

	Description
	-----------
    This function sends the specified simply blue command bytes over usart.
    The routine is a blocking call except for SPP_SEND_DATA command. The routine
    either will timeout or will be unblocked as soon as simply blue software
    receives and processes corresponding CFM message for the simply blue command
    being sent. SbProcessFrame routine unblocks the pending event.

	Arguments
	---------
    SbCommand : simply blue command bytes  
    Size      : size of simply blue command frame

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbSendCommand(uint8* SbCommand, uint16 Size)
{
    uint8  err;
    void   *msg;
    uint16 checksum;

    SbCommand[0]    = STX;
    SbCommand[1]    = REQ;
    checksum        = SbCommand[1] + SbCommand[2] + SbCommand[3] + SbCommand[4];
    SbCommand[5]    = checksum % 256;
	
    SbCommand[Size - 1] = ETX;
    
    if (usart_tx(SB_UART_PORT, SbCommand, Size) == Size) {
        if (SbCommand[2] == SPP_SEND_DATA) {
            return SBSTATUS_OK;
        }
    	msg = OSMboxPend(SbDevInfo.SbCmdMbox, BTCORE_CALLBACK_TIMEOUT, &err);
        if (err == OS_NO_ERR) {
            if ((uint32)msg == SBSTATUS_OK) {
                return SBSTATUS_OK;
            }
            else {
                return SBSTATUS_ERROR;
            }
        }
        else {
            return SBSTATUS_TIMEOUT;
        }
    }
	else {
		return SBSTATUS_UART_INCOMPLETE_TRANSFER;
	}
}

/******************************************************************************  
    Function: SbProcessFrame

	Description
	-----------
    This function is being called from SbTimerTask to process the received complete
    simply blue packet.This routine unblocks the pending events to denote a simply command
    is confirmed and completed.

	Arguments
	---------
    None.

    Returns
	-------
    None.
	
*******************************************************************************/
void SbProcessFrame(void)
{
    int    i;
    uint16 ServiceResponseStart=0;
    uint32 status;

    switch (SbEvent.bOpcode) 
    {
        case GAP_DEVICE_FOUND:              /* to be implemented by the user */ return; 
        case SPP_LINK_ESTABLISHED:          /* to be implemented by the user */ return;
        case SPP_INCOMING_LINK_ESTABLISHED: /* to be implemented by the user */ return;
	case SPP_LINK_RELEASED:             /* to be implemented by the user */ return;
        case GAP_ESTABLISH_SCO_LINK:        /* to be implemented by the user */ return;
        case GAP_RELEASE_SCO_LINK:          /* to be implemented by the user */ return;
	case SPP_SEND_DATA:                 /* to be implemented by the user */ return;
    	case SPP_INCOMING_DATA:             /* to be implemented by the user */ return;
	case SDAP_SERVICE_BROWSE:           /* to be implemented by the user */ return;
	case SDAP_SERVICE_REQUEST:          /* to be implemented by the user */ return;
	case MODULE_READY:
            if ( COMMON_MboxWaitingTasks( SbDevInfo.SbCmdMbox ) != 0 ) {
                OSMboxPost(SbDevInfo.SbCmdMbox, (void*)1);
            }
            /* At this point, the Simply Blue device is reset and ready to work */
            return;
        default:
            break;
	}
    
    if (SbEvent.bType == CFM) {
        //signal that sb command is completed
        status = ((!SbEvent.pPayload[0]) ? SBSTATUS_OK : SbEvent.pPayload[0]);
        OSMboxPost(SbDevInfo.SbCmdMbox, (void*)status);
    }
}

/******************************************************************************  
    Function: SbEstablishLink

	Description
	-----------
    This function initates establishment of ACL link with remote device that is 
    specified by passed in Bd Address. The serial port link is established between 
    specified local port and remote port on the local and remote devices. 
    
    This function waits on Mbox events untill simply blue CFM message and 
    link_established messages are processed.

	Arguments
	---------
    BdAddr       : Bd Address of the remote device
    LocalPortNo  : port number of the local device
    RemotePortNo : port number of the remote device

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbEstablishLink(BdAddrType BdAddr,
                           uint8 LocalPortNo, 
                           uint8 RemotePortNo)  
{
    uint16 payloadlen;
    uint8  SbCommand[15];
    int    i;
    uint8  err;
    void   *msg;

    payloadlen = 0x0008;

    SbCommand[2] = SPP_ESTABLISH_LINK; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    SbCommand[6] = LocalPortNo;           // Localport
    
    for (i = 0; i < 6; i++) {
        SbCommand[7 + i] = BdAddr[i];     // BDA
    }
    
    SbCommand[7 + i] = RemotePortNo;      // Remoteport

    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        //wait till link is established
    	msg = OSMboxPend(SbDevInfo.SbCmdMbox, BTCORE_CALLBACK_TIMEOUT, &err);
        if ( err == OS_NO_ERR && ((uint32)(msg) == SBSTATUS_OK)) {
            return SBSTATUS_OK;
        }
    }

    return SBSTATUS_ERROR;
}

/******************************************************************************  
    Function: SbReleaseLink

	Description
	-----------
    This function initates release of SPP link associated with the port number specified
    on the local device.
    
    This function waits on Mbox events untill simply blue CFM message and 
    link_released messages are processed.

	Arguments
	---------
    LocalPortNo  : port number of the local device

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbReleaseLink(uint8 LocalPortNo)  
{
    uint16 payloadlen;
    uint8  SbCommand[8];
    uint8  err;
    void   *msg;

    payloadlen = 0x0001;

    SbCommand[2] = SPP_RELEASE_LINK; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    SbCommand[6] = LocalPortNo; 
	
    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        //wait till link is released
    	msg = OSMboxPend(SbDevInfo.SbCmdMbox, BTCORE_CALLBACK_TIMEOUT, &err);
        if ( err == OS_NO_ERR && ((uint32)(msg) == SBSTATUS_OK)) {
            return SBSTATUS_OK;
        }
    }

    return SBSTATUS_ERROR;
}


/****************************************************************************
    Function: SbEnterTransparentMode

	Description
	-----------
    This function switches an active link to transparent mode.
    This function sends the "send transparent mode" command to the module

	Arguments
	---------
    LocalPortNo  : port number of the local device 

    Returns
	-------
    A SBStatus_T value indicating result of the operation.

****************************************************************************/
SBStatus_T SbEnterTransparentMode(uint8 LocalPortNo)
{

    int16 payloadlen;
    uint8 SbCommand[8];
    uint8 err;
    void  *msg;

    payloadlen = 0x0001;

    SbCommand[2] = SPP_TRANSPARENT_MODE; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    SbCommand[6] = LocalPortNo; 
	
    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        Transparent_flag = TRUE;          // transparent mode is now active     
        return SBSTATUS_OK;
    }

    return SBSTATUS_ERROR;
}

/****************************************************************************
    Function: SbSendUartBreak

	Description
	-----------
    This function switches the module to command mode.
    This function sends a UART Break to leave transparent mode.

	Arguments
	---------
    None.

    Returns
	-------
    None. 

****************************************************************************/
void SbSendUartBreak(void)
{
    uint32 ticks,new_tick=0;
    usart_t* usart = usart_tab[SB_UART_PORT];

    OS_ENTER_CRITICAL();

    UMDSL1 |= UBRK;                  // asserts TX line to 0
    ticks = OSTimeGet();
    while(OSTimeGet() < (ticks+1));  // wait for 10 ms (1 tick)
	UMDSL1 &= (~UBRK);           // asserts TX line to 1

    Transparent_flag = FALSE;        // command mode active

    OS_EXIT_CRITICAL();
}

/******************************************************************************
    Function: SbSdapStartSession

	Description
	-----------
    This function start an SDP session with a specified Bluetooth device and
	reports the result.

	Arguments
	---------
	BdAddr:
	The BD address of the device with which SDP session is to be started.

	Returns
	-------
	A boolean flag indicating whether the SDP session has been successfully
	established.
*******************************************************************************/
boolean SbSdapStartSession( BdAddrType bdAddr )
{
    uint16 payloadlen;
    uint8  SbCommand[13];
    int    i;

    payloadlen = 0x0006;

    SbCommand[2] = SDAP_CONNECT; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    for (i=0;i<6;i++)
        SbCommand[6 + i] = bdAddr[i];

    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        return TRUE;
    }

    return FALSE;
}

/******************************************************************************
    Function: SbSdapStopSession

	Description
	-----------
    This function stops the current SDP session and reports the result.

	Arguments
	---------
	None.

	Returns
	-------
	A boolean flag indicating whether the SDP session has been successfully
	terminated.
*******************************************************************************/
boolean SbSdapStopSession(void)
{
    uint16 payloadlen;
    uint8  SbCommand[7];

    payloadlen = 0x0;

    SbCommand[2] = SDAP_DISCONNECT; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion
	
    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        return TRUE;
    }
    return FALSE;
}

// service discovery(sdap) api
//
/******************************************************************************  
    Function: SbDeleteSdpRecords

	Description
	-----------
    This function deletes all the SDP records on the local device.

	Arguments
	---------
    None.

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbDeleteSdpRecords(void)
{
    uint16 payloadlen;
    uint8  SbCommand[7];

    payloadlen = 0x0000;

    SbCommand[2] = DELETE_SDP_RECORDS; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbStoreSdpRecord

	Description
	-----------
    This function stores a SDP record on the local device.

	Arguments
	---------
    entry  : sdp record bytes
    length : length of the sdp record

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbStoreSdpRecord(uint8* entry, uint16 length)
{
    uint16 i;
    uint16 payloadlen;
    uint8  SbCommand[340];

    payloadlen = length;

    SbCommand[2] = STORE_SDP_RECORD; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    for (i=0;i<length;i++)
        SbCommand[6 + i] = entry[i];	

    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbSetOperationMode

	Description
	-----------
    This function sets simply blue device operation mode.Please check simply blue
    software documentation for the details.

	Arguments
	---------
    mode  : simply blue operation mode that follows SBOperationMode_T enum.

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbSetOperationMode(SBOperationMode_T mode)
{
    uint16 payloadlen;
    uint8  SbCommand[8];

    payloadlen = 0x0001;

    SbCommand[2] = WRITE_OPERATION_MODE; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion
    SbCommand[6] = mode;

    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbNvsReadBdAddr

	Description
	-----------
    This function gets the local device's Bluetooth Device Address

	Arguments
	---------
    BdAddr : BD address of the local device will be fetched into this parameter. 
             
	
    Returns
	-------
	None.
	
*******************************************************************************/
void SbNvsReadBdAddr(BdAddrType BdAddr)
{
    uint16 payloadlen;
    uint8  SbCommand[7];

    payloadlen = 0x0000;

    SbCommand[2] = GAP_READ_LOCAL_BDA; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
        int i;
	    
        for (i=1;i<7;i++) {
	        BdAddr[i-1]=SbEvent.pPayload[i];
        }
    }
}

/******************************************************************************  
    Function: SbNvsWriteBdAddr

	Description
	-----------
    This function writes the local device's Bluetooth Device Address

	Arguments
	---------
    BdAddr : BD address of the local device will be fetched into this parameter. 
             
	
    Returns
	-------
	None.
	
*******************************************************************************/
void SbNvsWriteBdAddr(BdAddrType BdAddr)
{
    int i;
    uint16 payloadlen;
    uint8  SbCommand[13];

    payloadlen = 0x0006;

    SbCommand[2] = CHANGE_LOCAL_BDADDRESS; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion
    for(i=0;i<6;i++)
    {
        SbCommand[6+i] = BdAddr[5-i];
    }

    SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbNvsReadDeviceName

	Description
	-----------
    This function gets the local device name

	Arguments
	---------
    Name          : Name of the local device will be fetched in to this parameter. The applications
                    should make sure to pass in the same size as BtNameType specifies in sb.h
    
    MaxNameLength : The actual length of the local device name that is being received.

    Offset        : unused parameter
           
	
    Returns
	-------
	Length of the device name
	
*******************************************************************************/
uint8 SbNvsReadDeviceName(BtNameType Name, uint8 *MaxNameLength, uint8 Offset)
{
    uint16 payloadlen;
    uint8  SbCommand[7];
    uint8  len;
    int    i;

    payloadlen = 0x0000;

    SbCommand[2] = GAP_READ_LOCAL_NAME; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    Offset = Offset;
    len    = 0;

    if (SbSendCommand(SbCommand, 7 + payloadlen) == SBSTATUS_OK) {
    	len=SbEvent.pPayload[1];
        for (i=0;i<len;i++) {
    		Name[i]=SbEvent.pPayload[i+2];
        }
        *MaxNameLength = len;
    }

    return len;
}

/******************************************************************************  
    Function: SbNvsWriteDeviceName

	Description
	-----------
    This function writes the local device name in the NVS.

	Arguments
	---------
    Name          : Name of the local device will be fetched in to this parameter. The applications
                    should make sure to pass in the same size as BtNameType specifies in sb.h
    
    Returns
	-------
	Status of the command, TRUE command successfull, FALSE command failed
	
*******************************************************************************/
void SbNvsWriteDeviceName(BtNameType Name,uint8 length) 
{
    uint16 payloadlen;
    uint8  SbCommand[7+BT_NAME_LENGTH];
    int    i=0;

    while(Name[i] != '\0')
    {
        SbCommand[7+i] = Name[i];
        i++;
    }
    SbCommand[7+i] = '\0';
    payloadlen = length + 1;   // Payload length

    SbCommand[2] = GAP_WRITE_LOCAL_NAME; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion
    SbCommand[6] = length;

    SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbSetNvsDefaultAudioSettings

	Description
	-----------
    This function sets the default audio settings in Simply Blue NVS storage. These
    settings are used while creating bluetooth audio link, to configure AAI bus and any external
    audio codec boards connected to simply blue board.

	Arguments
	---------
    CodecType : valid values follow SbCodecType_T enum type
    AirFormat : valid values follow SbAirFormat_T enum type.
	
    Returns
	-------
	A SBStatus_T value indicating whether default audio settings are stored successfully in 
	simply Blue NVS or not.
*******************************************************************************/
SBStatus_T SbSetNvsDefaultAudioSettings(uint8 CodecType, uint8 AirFormat)
{
    uint16 payloadlen;
    uint8  SbCommand[9];

    payloadlen = 0x0002;

    SbCommand[2] = SET_DEFAULT_AUDIO_CONFIG; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    SbCommand[6] = CodecType;
    SbCommand[7] = AirFormat;
	
    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbResetFactorySettings

	Description
	-----------
    This function initializes the simply blue device opeartion with factory settings
    please check simply blue software documentation for the details.

	Arguments
	---------
    None  : 

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbResetFactorySettings(void)
{
    uint16 payloadlen;
    uint8  SbCommand[7];

    payloadlen = 0x0000;

    SbCommand[2] = RESTORE_FACTORY_SETTINGS; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbResetDevice

	Description
	-----------
    This function does Software RESET on local simply blue device.Please check simply blue
    software documentation for the details.

	Arguments
	---------
    None   

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbResetDevice(void)
{
    uint16 payloadlen;
    uint8  SbCommand[7];

    payloadlen = 0x0000;

    SbCommand[2] = RESET; 
    SbCommand[3] = (payloadlen & 0x00FF); // payload size is stored 
    SbCommand[4] = (payloadlen >> 8);     // in little endian fashion

    return SbSendCommand(SbCommand, 7 + payloadlen);
}

/******************************************************************************  
    Function: SbGetPacketState

	Description
	-----------
    This function returns the current simply blue(sb) packet processing state.
    A value of SBPACKET_ERR means an error occured in sb packet reception. No
    error recovery is being implemented. Applications should call this function
    and if SBPACKET_ERR happens, sb software stops processing further sb packets
    at this time.
    

	Arguments
	---------
	EventP:
	A pointer to the Connection_Info_Event_Type

	Returns
	-------
	A SbPacketState_T value indicating current state of sb packet processing.

*******************************************************************************/
SbPacketState_T SbGetPacketState(void)
{
    return SbDevInfo.SbPacketState;
}

/******************************************************************************  
    Function: SbTimerTask

	Description
	-----------
    This function is being invoked repeatedly for every SB_TIMER_TICKS, process
    the received bytes from Usart that is connected to SB hardware's Usart.
    A value of SBPACKET_ERR means an error occured in sb packet reception. No
    error recovery is being implemented. Applications should call SbGetPacketState
    function and if SBPACKET_ERR happens, sb software stops processing further 
    sb packets at this time.
    
    NOTE: Simply Blue transparent mode is not implemented. This routine is currently
    applicable only for Command mode.

	Arguments
	---------
	None

	Returns
	-------
	None.

*******************************************************************************/
void SbTimerTask( void )
{
    uint16 rxCount;
    uint8  in;
    uint16 DataLen;
    uint8  cs;
	
    while ((rxCount = usart_rx_count(SB_UART_PORT)) > 0) {
        switch(SbDevInfo.SbPacketState) {
        case SBPACKET_START:
            usart_rx(SB_UART_PORT, &in, 1);
            rxCount--;
            if (in == 0x2) {
                SbDevInfo.SbPacketState = SBPACKET_TYPE;
            }
            else {
                SbDevInfo.SbPacketState = SBPACKET_ERR;
            }
            break;
        case SBPACKET_TYPE:
            usart_rx(SB_UART_PORT, &in, 1);
            rxCount--;
            if (in == CFM || in == IND || in == RES) {
                SbDevInfo.SbPacketState = SBPACKET_OPCODE;
                SbEvent.bType   = in;
            }
            else {
                SbDevInfo.SbPacketState = SBPACKET_ERR;
            }
            break;
        case SBPACKET_OPCODE:
            usart_rx(SB_UART_PORT, &in, 1);
            rxCount--;
            SbEvent.bOpcode = in;
            SbDevInfo.SbPacketState = SBPACKET_DATALENGTH;
            break;
        case SBPACKET_DATALENGTH:
            if (rxCount >= 2) {
                usart_rx(SB_UART_PORT, (uint8*)&DataLen, 2);
                rxCount -=2;
                SbEvent.wPayloadlen = DataLen;
                SbDevInfo.SbPacketState = SBPACKET_CHECKSUM;
            }
            else {
                return; //wait for some more bytes to receive
            }
            break;
        case SBPACKET_CHECKSUM:
            usart_rx(SB_UART_PORT, &in, 1);
            rxCount--;
            cs = SbEvent.bType + SbEvent.bOpcode + 
                 (SbEvent.wPayloadlen >> 8) + (SbEvent.wPayloadlen & 0x00FF);
            if (cs == in) {
                SbDevInfo.SbPacketState = SBPACKET_DATA;
            }
            else {
                SbDevInfo.SbPacketState = SBPACKET_ERR;
            }
            break;
        case SBPACKET_DATA:
            if (rxCount >= (SbEvent.wPayloadlen - SbEvent.wPayloadOffset)) {
                usart_rx(SB_UART_PORT, &SbEvent.pPayload[SbEvent.wPayloadOffset], 
                         SbEvent.wPayloadlen - SbEvent.wPayloadOffset);
                rxCount -=(SbEvent.wPayloadlen - SbEvent.wPayloadOffset);                         
                SbDevInfo.SbPacketState = SBPACKET_END;
                SbEvent.wPayloadOffset = 0;
            }
            else {
                usart_rx(SB_UART_PORT, &SbEvent.pPayload[SbEvent.wPayloadOffset], 
                         rxCount);
                SbEvent.wPayloadOffset = rxCount;
                rxCount = 0;
            }
            break;
        case SBPACKET_END:
            usart_rx(SB_UART_PORT, &in, 1);
            if (in == ETX) {
                SbProcessFrame();    /* process the frame once received */
                SbDevInfo.SbPacketState = SBPACKET_START;
                //return;
            }
            else {
                SbDevInfo.SbPacketState = SBPACKET_ERR;
            }
            break;
        default:
            // invalid Packet received from SB usart,skipping of bytes
            // to next valid packet is not incorporated. simply blue assumes
            // error free usart transport. No further processing.
            // apps should call SbGetPacketState() and do proper exit or
            // analyze on error state.
            SbDevInfo.SbPacketState = SBPACKET_ERR;
            return;
        }
    }
}

/******************************************************************************  
    Function: SbConfigureLocalDevice

	Description
	-----------
    This function resets the simply blue device to factory settings first, next
    deletes all sdp records, sets operation mode as Command mode, and finally does 
    Software RESET on local simply blue device.Please check simply blue
    software documentation for the details of these operations.

	Arguments
	---------
    None   

    Returns
	-------
    A SBStatus_T value indicating result of the operation.
	
*******************************************************************************/
SBStatus_T SbConfigureLocalDevice(void)
{
    SBStatus_T status;
    //Initialize audio codec settings
    SbCodecType_T CodecType;
    SbAirFormat_T AirFormat;

    CodecType = SBCODEC_OKI;
    AirFormat = SBAIRFMT_CVSD;

    status = SbResetFactorySettings();
    
    if (status == SBSTATUS_OK) {
        status = SbDeleteSdpRecords();
        if (status == SBSTATUS_OK) {
            status = SbSetOperationMode(SB_COMMAND_MODE);
            if (status == SBSTATUS_OK) {
                status = SbSetNvsDefaultAudioSettings(CodecType, AirFormat);
                if (status == SBSTATUS_OK) {
                    status = SbResetDevice();
                }
            }
        }
    }
    return status;
}


